@@ -26,7 +26,12 @@
 #include <errno.h>
 #include <locale.h>
 #include <stdlib.h>
+
+#ifdef HAVE_WEBKITGTK_6_0
+#include <webkit/webkit.h>
+#else
 #include <webkit2/webkit2.h>
+#endif
 
 #include "cc-common-language.h"
 #include "gis-assistant.h"
@@ -70,7 +75,7 @@ typedef enum {
 static GParamSpec *obj_props[PROP_AVATAR + 1];
 
 struct _GisDriver {
-  GtkApplication  parent_instance;
+  AdwApplication  parent_instance;
 
   GtkWindow *main_window;
   GisAssistant *assistant;
@@ -91,7 +96,7 @@ struct _GisDriver {
   gchar *username;
   gchar *full_name;  /* (owned) (nullable) */
 
-  GdkPixbuf *avatar;  /* (owned) (nullable) */
+  GdkPaintable *avatar;  /* (owned) (nullable) */
 
   GisDriverMode mode;
   UmAccountMode account_mode;
@@ -103,7 +108,7 @@ struct _GisDriver {
   GKeyFile *vendor_conf_file;
 };
 
-G_DEFINE_TYPE (GisDriver, gis_driver, GTK_TYPE_APPLICATION)
+G_DEFINE_TYPE (GisDriver, gis_driver, ADW_TYPE_APPLICATION)
 
 static void
 gis_driver_dispose (GObject *object)
@@ -155,12 +160,12 @@ prepare_main_window (GisDriver *driver)
 {
   GtkWidget *child, *sw;
 
-  child = g_object_ref (gtk_bin_get_child (GTK_BIN (driver->main_window)));
-  gtk_container_remove (GTK_CONTAINER (driver->main_window), child);
-  sw = gtk_scrolled_window_new (NULL, NULL);
-  gtk_widget_show (sw);
-  gtk_container_add (GTK_CONTAINER (driver->main_window), sw);
-  gtk_container_add (GTK_CONTAINER (sw), child);
+  child = gtk_window_get_child (GTK_WINDOW (driver->main_window));
+  g_object_ref (child);
+  gtk_window_set_child (GTK_WINDOW (driver->main_window), NULL);
+  sw = gtk_scrolled_window_new ();
+  gtk_window_set_child (GTK_WINDOW (driver->main_window), sw);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (sw), child);
   g_object_unref (child);
 
   g_signal_connect_swapped (driver->assistant,
@@ -299,11 +304,11 @@ gis_driver_get_full_name (GisDriver *driver)
  * Since: 3.36
  */
 void
-gis_driver_set_avatar (GisDriver *driver,
-                       GdkPixbuf *avatar)
+gis_driver_set_avatar (GisDriver    *driver,
+                       GdkPaintable *avatar)
 {
   g_return_if_fail (GIS_IS_DRIVER (driver));
-  g_return_if_fail (avatar == NULL || GDK_IS_PIXBUF (avatar));
+  g_return_if_fail (avatar == NULL || GDK_IS_PAINTABLE (avatar));
 
   if (g_set_object (&driver->avatar, avatar))
     g_object_notify_by_pspec (G_OBJECT (driver), obj_props[PROP_AVATAR]);
@@ -318,7 +323,7 @@ gis_driver_set_avatar (GisDriver *driver,
  * Returns: (nullable) (transfer none): avatar of the main user, or %NULL if not known
  * Since: 3.36
  */
-GdkPixbuf *
+GdkPaintable *
 gis_driver_get_avatar (GisDriver *driver)
 {
   g_return_val_if_fail (GIS_IS_DRIVER (driver), NULL);
@@ -677,41 +682,22 @@ gis_driver_activate (GApplication *app)
   gtk_window_present (GTK_WINDOW (driver->main_window));
 }
 
-static void
-set_small_screen_based_on_primary_monitor (GisDriver *driver)
-{
-  GdkDisplay *default_display;
-  GdkMonitor *primary_monitor;
-
-  default_display = gdk_display_get_default ();
-  if (default_display == NULL)
-    return;
-
-  primary_monitor = gdk_display_get_primary_monitor (default_display);
-  if (primary_monitor == NULL)
-    return;
-
-  driver->small_screen = monitor_is_small (primary_monitor);
-}
-
 /* Recompute driver->small_screen based on the monitor where the window is
  * located, if the window is actually realized. If not, recompute it based on
  * the primary monitor of the default display. */
 static void
-recompute_small_screen (GisDriver *driver) {
-  GdkWindow *window;
-  GdkDisplay *default_display = gdk_display_get_default ();
+recompute_small_screen (GisDriver *driver)
+{
   GdkMonitor *active_monitor;
   gboolean old_value = driver->small_screen;
 
-  if (!gtk_widget_get_realized (GTK_WIDGET (driver->main_window)))
+  if (gtk_widget_get_realized (GTK_WIDGET (driver->main_window)))
     {
-      set_small_screen_based_on_primary_monitor (driver);
-    }
-  else
-    {
-      window = gtk_widget_get_window (GTK_WIDGET (driver->main_window));
-      active_monitor = gdk_display_get_monitor_at_window (default_display, window);
+      GdkDisplay *default_display = gdk_display_get_default ();
+      GdkSurface *surface;
+
+      surface = gtk_native_get_surface (GTK_NATIVE (driver->main_window));
+      active_monitor = gdk_display_get_monitor_at_surface (default_display, surface);
       driver->small_screen = monitor_is_small (active_monitor);
     }
 
@@ -722,8 +708,6 @@ recompute_small_screen (GisDriver *driver) {
 static void
 update_screen_size (GisDriver *driver)
 {
-  GdkWindow *window;
-  GdkGeometry size_hints;
   GtkWidget *sw;
 
   recompute_small_screen (driver);
@@ -731,55 +715,34 @@ update_screen_size (GisDriver *driver)
   if (!gtk_widget_get_realized (GTK_WIDGET (driver->main_window)))
     return;
 
-  sw = gtk_bin_get_child (GTK_BIN (driver->main_window));
-  window = gtk_widget_get_window (GTK_WIDGET (driver->main_window));
+  sw = gtk_window_get_child (GTK_WINDOW (driver->main_window));
 
   if (driver->small_screen)
     {
-      if (window)
-        gdk_window_set_functions (window,
-                                  GDK_FUNC_ALL | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE);
-
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
-
-      gtk_window_set_geometry_hints (driver->main_window, NULL, NULL, 0);
+      gtk_window_set_default_size (driver->main_window, -1, -1);
       gtk_window_set_resizable (driver->main_window, TRUE);
-      gtk_window_set_position (driver->main_window, GTK_WIN_POS_NONE);
-
       gtk_window_maximize (driver->main_window);
       gtk_window_present (driver->main_window);
     }
   else
     {
-      if (window)
-        gdk_window_set_functions (window,
-                                  GDK_FUNC_ALL | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE |
-                                  GDK_FUNC_RESIZE | GDK_FUNC_MOVE | GDK_FUNC_MAXIMIZE);
-
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                       GTK_POLICY_NEVER,
                                       GTK_POLICY_NEVER);
-
-      size_hints.min_width = size_hints.max_width = 1024;
-      size_hints.min_height = size_hints.max_height = 768;
-      size_hints.win_gravity = GDK_GRAVITY_CENTER;
-
-      gtk_window_set_geometry_hints (driver->main_window,
-                                     NULL,
-                                     &size_hints,
-                                     GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_WIN_GRAVITY);
+      gtk_window_set_default_size (driver->main_window, 1024, 768);
       gtk_window_set_resizable (driver->main_window, FALSE);
-      gtk_window_set_position (driver->main_window, GTK_WIN_POS_CENTER_ALWAYS);
-
       gtk_window_unmaximize (driver->main_window);
       gtk_window_present (driver->main_window);
     }
 }
 
 static void
-screen_size_changed (GdkScreen *screen, GisDriver *driver)
+on_surface_enter_monitor_cb (GdkSurface *surface,
+                             GdkMonitor *monitor,
+                             GisDriver  *driver)
 {
   update_screen_size (driver);
 }
@@ -787,7 +750,15 @@ screen_size_changed (GdkScreen *screen, GisDriver *driver)
 static void
 window_realize_cb (GtkWidget *widget, gpointer user_data)
 {
-  update_screen_size (GIS_DRIVER (user_data));
+  GdkSurface *surface;
+  GisDriver *driver;
+
+  driver = GIS_DRIVER (user_data);
+
+  surface = gtk_native_get_surface (GTK_NATIVE (widget));
+  g_signal_connect (surface, "enter-monitor", G_CALLBACK (on_surface_enter_monitor_cb), driver);
+
+  update_screen_size (driver);
 }
 
 static void
@@ -824,7 +795,6 @@ gis_driver_startup (GApplication *app)
 
   driver->main_window = g_object_new (GTK_TYPE_APPLICATION_WINDOW,
                                     "application", app,
-                                    "type", GTK_WINDOW_TOPLEVEL,
                                     "icon-name", "preferences-system",
                                     "deletable", FALSE,
                                     NULL);
@@ -835,9 +805,8 @@ gis_driver_startup (GApplication *app)
                     (gpointer)app);
 
   driver->assistant = g_object_new (GIS_TYPE_ASSISTANT, NULL);
-  gtk_container_add (GTK_CONTAINER (driver->main_window), GTK_WIDGET (driver->assistant));
-
-  gtk_widget_show (GTK_WIDGET (driver->assistant));
+  gtk_window_set_child (GTK_WINDOW (driver->main_window),
+                        GTK_WIDGET (driver->assistant));
 
   gis_driver_set_user_language (driver, setlocale (LC_MESSAGES, NULL), FALSE);
 
@@ -848,17 +817,7 @@ gis_driver_startup (GApplication *app)
 static void
 gis_driver_init (GisDriver *driver)
 {
-  GdkScreen *screen;
-
-  screen = gdk_screen_get_default ();
-
-  set_small_screen_based_on_primary_monitor (driver);
-
   load_vendor_conf_file (driver);
-
-  if (screen != NULL)
-    g_signal_connect (screen, "size-changed",
-                      G_CALLBACK (screen_size_changed), driver);
 }
 
 static void
@@ -948,7 +907,7 @@ gis_driver_class_init (GisDriverClass *klass)
     g_param_spec_object ("avatar",
                          "Avatar",
                          "Avatar of the main user.",
-                         GDK_TYPE_PIXBUF,
+                         GDK_TYPE_PAINTABLE,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (gobject_class, G_N_ELEMENTS (obj_props), obj_props);
