@@ -88,6 +88,21 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+static void
+clear_password_validation_error (GtkWidget *entry)
+{
+  gtk_widget_remove_css_class (entry, "error");
+  gtk_widget_set_tooltip_text (entry, NULL);
+}
+
+static void
+set_password_validation_error (GtkWidget   *entry,
+                               const gchar *text)
+{
+  gtk_widget_add_css_class (entry, "error");
+  gtk_widget_set_tooltip_text (entry, text);
+}
+
 static void
 validation_changed (GisAccountPageEnterprise *page)
 {
@@ -123,6 +138,9 @@ show_error_dialog (GisAccountPageEnterprise *page,
                                               "%s", error->message);
   }
 
+  g_signal_connect (dialog, "response",
+                    G_CALLBACK (gtk_window_destroy),
+                    NULL);
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
@@ -313,7 +331,7 @@ join_show_prompt (GisAccountPageEnterprise *page,
     gtk_editable_set_text (GTK_EDITABLE (page->join_computer), hostname);
 
   clear_entry_validation_error (GTK_ENTRY (page->join_name));
-  clear_entry_validation_error (GTK_ENTRY (page->join_password));
+  clear_password_validation_error (page->join_password);
 
   if (!page->join_prompted) {
     name = um_realm_kerberos_membership_get_suggested_administrator (membership);
@@ -330,7 +348,7 @@ join_show_prompt (GisAccountPageEnterprise *page,
 
   } else if (g_error_matches (error, UM_REALM_ERROR, UM_REALM_ERROR_BAD_PASSWORD)) {
     g_debug ("Bad admin password: %s", error->message);
-    set_entry_validation_error (GTK_ENTRY (page->join_password), error->message);
+    set_password_validation_error (page->join_password, error->message);
 
   } else {
     g_debug ("Admin login failure: %s", error->message);
@@ -468,7 +486,7 @@ on_realm_login (GObject *source,
 
   } else if (g_error_matches (error, UM_REALM_ERROR, UM_REALM_ERROR_BAD_PASSWORD)) {
     g_debug ("Problem with the user's password: %s", error->message);
-    set_entry_validation_error (GTK_ENTRY (page->password), error->message);
+    set_password_validation_error (page->password, error->message);
     gtk_widget_grab_focus (page->password);
     apply_complete (page, FALSE);
 
@@ -718,7 +736,7 @@ on_domain_changed (GtkComboBox *widget,
 
   page->domain_chosen = TRUE;
   validation_changed (page);
-  clear_entry_validation_error (GTK_ENTRY (adw_bin_get_child (ADW_BIN (widget))));
+  clear_entry_validation_error (GTK_ENTRY (gtk_combo_box_get_child (widget)));
 }
 
 static void
@@ -730,6 +748,13 @@ on_entry_changed (GtkEditable *editable,
   clear_entry_validation_error (GTK_ENTRY (editable));
 }
 
+static void
+on_password_changed (GtkEditable *editable,
+                     gpointer user_data)
+{
+  clear_password_validation_error (GTK_WIDGET (editable));
+}
+
 static void
 gis_account_page_enterprise_realize (GtkWidget *widget)
 {
@@ -764,6 +789,10 @@ gis_account_page_enterprise_constructed (GObject *object)
                     G_CALLBACK (on_domain_changed), page);
   g_signal_connect (page->login, "changed",
                     G_CALLBACK (on_entry_changed), page);
+  g_signal_connect (page->password, "changed",
+                    G_CALLBACK (on_password_changed), page);
+  g_signal_connect (page->join_password, "changed",
+                    G_CALLBACK (on_password_changed), page);
 }
 
 static void
