@@ -42,7 +42,6 @@ struct _CcLanguageChooserPrivate
         GtkWidget *filter_entry;
         GtkWidget *language_list;
 
-        GtkWidget *scrolled_window;
         GtkWidget *no_results;
         GtkWidget *more_item;
 
@@ -90,11 +89,11 @@ static GtkWidget *
 padded_label_new (char *text)
 {
         GtkWidget *widget;
-        widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+        widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
         gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
-        gtk_widget_set_margin_top (widget, 10);
-        gtk_widget_set_margin_bottom (widget, 10);
-        gtk_box_pack_start (GTK_BOX (widget), gtk_label_new (text), FALSE, FALSE, 0);
+        gtk_widget_set_margin_top (widget, 12);
+        gtk_widget_set_margin_bottom (widget, 12);
+        gtk_box_append (GTK_BOX (widget), gtk_label_new (text));
         return widget;
 }
 
@@ -143,22 +142,21 @@ language_widget_new (const char *locale_id,
         locale_current_name = gnome_get_language_from_locale (locale_id, NULL);
         locale_untranslated_name = gnome_get_language_from_locale (locale_id, "C");
 
-        widget->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-        gtk_widget_set_margin_top (widget->box, 10);
-        gtk_widget_set_margin_bottom (widget->box, 10);
-        gtk_widget_set_margin_start (widget->box, 10);
-        gtk_widget_set_margin_end (widget->box, 10);
+        widget->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+        gtk_widget_set_margin_top (widget->box, 12);
+        gtk_widget_set_margin_bottom (widget->box, 12);
+        gtk_widget_set_margin_start (widget->box, 12);
+        gtk_widget_set_margin_end (widget->box, 12);
         gtk_widget_set_halign (widget->box, GTK_ALIGN_FILL);
 
         label = gtk_label_new (language_name);
         gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
         gtk_label_set_max_width_chars (GTK_LABEL (label), 30);
         gtk_label_set_xalign (GTK_LABEL (label), 0);
-        gtk_box_pack_start (GTK_BOX (widget->box), label, FALSE, FALSE, 0);
+        gtk_box_append (GTK_BOX (widget->box), label);
 
-        widget->checkmark = gtk_image_new_from_icon_name ("object-select-symbolic", GTK_ICON_SIZE_MENU);
-        gtk_box_pack_start (GTK_BOX (widget->box), widget->checkmark, FALSE, FALSE, 0);
-        gtk_widget_show (widget->checkmark);
+        widget->checkmark = gtk_image_new_from_icon_name ("object-select-symbolic");
+        gtk_box_append (GTK_BOX (widget->box), widget->checkmark);
 
         if (country_name) {
                 label = gtk_label_new (country_name);
@@ -166,8 +164,9 @@ language_widget_new (const char *locale_id,
                 gtk_label_set_max_width_chars (GTK_LABEL (label), 30);
                 gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
                 gtk_label_set_xalign (GTK_LABEL (label), 0);
+                gtk_widget_set_hexpand (label, TRUE);
                 gtk_widget_set_halign (label, GTK_ALIGN_END);
-                gtk_box_pack_end (GTK_BOX (widget->box), label, FALSE, FALSE, 0);
+                gtk_box_append (GTK_BOX (widget->box), label);
         }
 
         widget->locale_id = g_strdup (locale_id);
@@ -188,33 +187,29 @@ language_widget_new (const char *locale_id,
         return widget->box;
 }
 
-static void
-sync_checkmark (GtkWidget *row,
-                gpointer   user_data)
-{
-        GtkWidget *child;
-        LanguageWidget *widget;
-        gchar *locale_id;
-        gboolean should_be_visible;
-
-        child = gtk_bin_get_child (GTK_BIN (row));
-        widget = get_language_widget (child);
-
-        if (widget == NULL)
-                return;
-
-        locale_id = user_data;
-        should_be_visible = g_str_equal (widget->locale_id, locale_id);
-        gtk_widget_set_opacity (widget->checkmark, should_be_visible ? 1.0 : 0.0);
-}
-
 static void
 sync_all_checkmarks (CcLanguageChooser *chooser)
 {
         CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+        GtkWidget *row;
 
-        gtk_container_foreach (GTK_CONTAINER (priv->language_list),
-                               sync_checkmark, priv->language);
+        row = gtk_widget_get_first_child (priv->language_list);
+        while (row) {
+                LanguageWidget *widget;
+                GtkWidget *child;
+                gboolean should_be_visible;
+
+                child = gtk_list_box_row_get_child (GTK_LIST_BOX_ROW (row));
+                widget = get_language_widget (child);
+
+                if (widget == NULL)
+                        return;
+
+                should_be_visible = g_str_equal (widget->locale_id, priv->language);
+                gtk_widget_set_opacity (widget->checkmark, should_be_visible ? 1.0 : 0.0);
+
+                row = gtk_widget_get_next_sibling (row);
+        }
 }
 
 static GtkWidget *
@@ -223,14 +218,18 @@ more_widget_new (void)
         GtkWidget *widget;
         GtkWidget *arrow;
 
-        widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+        widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
         gtk_widget_set_tooltip_text (widget, _("More…"));
 
-        arrow = gtk_image_new_from_icon_name ("view-more-symbolic", GTK_ICON_SIZE_MENU);
+        arrow = g_object_new (GTK_TYPE_IMAGE,
+                              "icon-name", "view-more-symbolic",
+                              "hexpand", TRUE,
+                              "halign", GTK_ALIGN_CENTER,
+                              NULL);
         gtk_style_context_add_class (gtk_widget_get_style_context (arrow), "dim-label");
-        gtk_widget_set_margin_top (widget, 10);
-        gtk_widget_set_margin_bottom (widget, 10);
-        gtk_box_pack_start (GTK_BOX (widget), arrow, TRUE, TRUE, 0);
+        gtk_widget_set_margin_top (widget, 12);
+        gtk_widget_set_margin_bottom (widget, 12);
+        gtk_box_append (GTK_BOX (widget), arrow);
 
         return widget;
 }
@@ -242,7 +241,7 @@ no_results_widget_new (void)
 
         widget = padded_label_new (_("No languages found"));
         gtk_widget_set_sensitive (widget, FALSE);
-        gtk_widget_show_all (widget);
+
         return widget;
 }
 
@@ -260,7 +259,7 @@ add_one_language (CcLanguageChooser *chooser,
 
 	widget = language_widget_new (locale_id, !is_initial);
         if (widget)
-                gtk_container_add (GTK_CONTAINER (priv->language_list), widget);
+                gtk_list_box_append (GTK_LIST_BOX (priv->language_list), widget);
 }
 
 static void
@@ -287,10 +286,8 @@ add_languages (CcLanguageChooser  *chooser,
                         add_one_language (chooser, locale_id, FALSE);
         }
 
-        gtk_container_add (GTK_CONTAINER (priv->language_list), priv->more_item);
+        gtk_list_box_append (GTK_LIST_BOX (priv->language_list), priv->more_item);
         gtk_list_box_set_placeholder (GTK_LIST_BOX (priv->language_list), priv->no_results);
-
-        gtk_widget_show_all (priv->language_list);
 }
 
 static void
@@ -315,7 +312,7 @@ language_visible (GtkListBoxRow *row,
         GtkWidget *child;
         const char *search_term;
 
-        child = gtk_bin_get_child (GTK_BIN (row));
+        child = gtk_list_box_row_get_child (row);
         if (child == priv->more_item)
                 return !priv->showing_extra;
 
@@ -324,7 +321,7 @@ language_visible (GtkListBoxRow *row,
         if (!priv->showing_extra && widget->is_extra)
                 return FALSE;
 
-        search_term = gtk_entry_get_text (GTK_ENTRY (priv->filter_entry));
+        search_term = gtk_editable_get_text (GTK_EDITABLE (priv->filter_entry));
         if (!search_term || !*search_term)
                 return TRUE;
 
@@ -354,8 +351,8 @@ sort_languages (GtkListBoxRow *a,
         LanguageWidget *la, *lb;
         int ret;
 
-        la = get_language_widget (gtk_bin_get_child (GTK_BIN (a)));
-        lb = get_language_widget (gtk_bin_get_child (GTK_BIN (b)));
+        la = get_language_widget (gtk_list_box_row_get_child (a));
+        lb = get_language_widget (gtk_list_box_row_get_child (b));
 
         if (la == NULL)
                 return 1;
@@ -389,12 +386,8 @@ show_more (CcLanguageChooser *chooser)
 {
         CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
 
-        gtk_widget_show (priv->filter_entry);
         gtk_widget_grab_focus (priv->filter_entry);
 
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->scrolled_window),
-					GTK_POLICY_NEVER,
-					GTK_POLICY_AUTOMATIC);
 	gtk_widget_set_valign (GTK_WIDGET (chooser), GTK_ALIGN_FILL);
 
         priv->showing_extra = TRUE;
@@ -441,7 +434,7 @@ row_activated (GtkListBox        *box,
         if (row == NULL)
                 return;
 
-        child = gtk_bin_get_child (GTK_BIN (row));
+        child = gtk_list_box_row_get_child (row);
         if (child == priv->more_item) {
                 show_more (chooser);
         } else {
@@ -455,23 +448,6 @@ row_activated (GtkListBox        *box,
         }
 }
 
-static void
-update_header_func (GtkListBoxRow *child,
-                    GtkListBoxRow *before,
-                    gpointer       user_data)
-{
-        GtkWidget *header;
-
-        if (before == NULL) {
-                gtk_list_box_row_set_header (child, NULL);
-                return;
-        }
-
-        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-        gtk_list_box_row_set_header (child, header);
-        gtk_widget_show (header);
-}
-
 static void
 cc_language_chooser_constructed (GObject *object)
 {
@@ -487,8 +463,6 @@ cc_language_chooser_constructed (GObject *object)
                                     sort_languages, chooser, NULL);
         gtk_list_box_set_filter_func (GTK_LIST_BOX (priv->language_list),
                                       language_visible, chooser, NULL);
-        gtk_list_box_set_header_func (GTK_LIST_BOX (priv->language_list),
-                                      update_header_func, chooser, NULL);
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (priv->language_list),
                                          GTK_SELECTION_NONE);
         add_all_languages (chooser);
@@ -563,7 +537,6 @@ cc_language_chooser_class_init (CcLanguageChooserClass *klass)
 
         gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, filter_entry);
         gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, language_list);
-        gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, scrolled_window);
 
 	object_class->finalize = cc_language_chooser_finalize;
         object_class->get_property = cc_language_chooser_get_property;
@@ -599,7 +572,7 @@ void
 cc_language_chooser_clear_filter (CcLanguageChooser *chooser)
 {
         CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
-        gtk_entry_set_text (GTK_ENTRY (priv->filter_entry), "");
+        gtk_editable_set_text (GTK_EDITABLE (priv->filter_entry), "");
 }
 
 const gchar *
