@@ -22,15 +22,19 @@
  */
 
 struct _GisLocationEntryPrivate {
+    GtkWidget        *entry;
     GWeatherLocation *location;
     GWeatherLocation *top;
     gboolean          show_named_timezones;
-    gboolean          custom_text;
     GCancellable     *cancellable;
     GtkTreeModel     *model;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GisLocationEntry, gis_location_entry, GTK_TYPE_SEARCH_ENTRY)
+static void editable_iface_init (GtkEditableInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (GisLocationEntry, gis_location_entry, GTK_TYPE_WIDGET,
+                         G_ADD_PRIVATE (GisLocationEntry)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE, editable_iface_init));
 
 enum {
     PROP_0,
@@ -86,6 +90,21 @@ static gboolean match_selected (GtkEntryCompletion *completion,
 static void     entry_changed (GisLocationEntry *entry);
 static void _no_matches (GtkEntryCompletion *completion, GisLocationEntry *entry);
 
+static GtkEditable*
+gis_location_entry_get_delegate (GtkEditable *editable)
+{
+    GisLocationEntry *entry = GIS_LOCATION_ENTRY (editable);
+    GisLocationEntryPrivate *priv = gis_location_entry_get_instance_private (entry);
+
+    return GTK_EDITABLE (priv->entry);
+}
+
+static void
+editable_iface_init (GtkEditableInterface *iface)
+{
+    iface->get_delegate = gis_location_entry_get_delegate;
+}
+
 static void
 gis_location_entry_init (GisLocationEntry *entry)
 {
@@ -94,9 +113,13 @@ gis_location_entry_init (GisLocationEntry *entry)
 
     priv = entry->priv = gis_location_entry_get_instance_private (entry);
 
+    priv->entry = gtk_entry_new ();
+    gtk_widget_set_parent (priv->entry, GTK_WIDGET (entry));
+    gtk_editable_init_delegate (GTK_EDITABLE (entry));
+
     completion = gtk_entry_completion_new ();
 
-    gtk_entry_completion_set_popup_set_width (completion, FALSE);
+    gtk_entry_completion_set_popup_set_width (completion, TRUE);
     gtk_entry_completion_set_text_column (completion, LOC_GIS_LOCATION_ENTRY_COL_DISPLAY_NAME);
     gtk_entry_completion_set_match_func (completion, matcher, NULL, NULL);
     gtk_entry_completion_set_inline_completion (completion, TRUE);
@@ -107,10 +130,9 @@ gis_location_entry_init (GisLocationEntry *entry)
     g_signal_connect (completion, "no-matches",
                       G_CALLBACK (_no_matches), entry);
 
-    gtk_entry_set_completion (GTK_ENTRY (entry), completion);
+    gtk_entry_set_completion (GTK_ENTRY (entry->priv->entry), completion);
     g_object_unref (completion);
 
-    priv->custom_text = FALSE;
     g_signal_connect (entry, "changed",
                       G_CALLBACK (entry_changed), NULL);
 }
@@ -146,6 +168,9 @@ dispose (GObject *object)
         priv->cancellable = NULL;
     }
 
+    gtk_editable_finish_delegate (GTK_EDITABLE (entry));
+    g_clear_pointer (&priv->entry, gtk_widget_unparent);
+
     G_OBJECT_CLASS (gis_location_entry_parent_class)->dispose (object);
 }
 
@@ -186,7 +211,7 @@ constructed (GObject *object)
     fill_location_entry_model (store, entry->priv->top, NULL, NULL, NULL, NULL, entry->priv->show_named_timezones);
 
     entry->priv->model = GTK_TREE_MODEL (store);
-    completion = gtk_entry_get_completion (GTK_ENTRY (entry));
+    completion = gtk_entry_get_completion (GTK_ENTRY (entry->priv->entry));
     gtk_entry_completion_set_match_func (completion, matcher, NULL, NULL);
     gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
 
@@ -197,6 +222,7 @@ static void
 gis_location_entry_class_init (GisLocationEntryClass *location_entry_class)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (location_entry_class);
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (location_entry_class);
 
     object_class->constructed = constructed;
     object_class->finalize = finalize;
@@ -226,6 +252,10 @@ gis_location_entry_class_init (GisLocationEntryClass *location_entry_class)
                              "The selected GWeatherLocation",
                              GWEATHER_TYPE_LOCATION,
                              G_PARAM_READWRITE));
+
+    gtk_editable_install_properties (object_class, LAST_PROP);
+
+    gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
 }
 
 static void
@@ -234,6 +264,9 @@ set_property (GObject *object, guint prop_id,
 {
     GisLocationEntry *entry = GIS_LOCATION_ENTRY (object);
 
+    if (gtk_editable_delegate_set_property (object, prop_id, value, pspec))
+      return;
+
     switch (prop_id) {
     case PROP_TOP:
         entry->priv->top = g_value_dup_object (value);
@@ -257,6 +290,9 @@ get_property (GObject *object, guint prop_id,
 {
     GisLocationEntry *entry = GIS_LOCATION_ENTRY (object);
 
+    if (gtk_editable_delegate_get_property (object, prop_id, value, pspec))
+      return;
+
     switch (prop_id) {
     case PROP_SHOW_NAMED_TIMEZONES:
         g_value_set_boolean (value, entry->priv->show_named_timezones);
@@ -276,26 +312,33 @@ entry_changed (GisLocationEntry *entry)
     GtkEntryCompletion *completion;
     const gchar *text;
 
-    completion = gtk_entry_get_completion (GTK_ENTRY (entry));
+    completion = gtk_entry_get_completion (GTK_ENTRY (entry->priv->entry));
 
     if (entry->priv->cancellable) {
         g_cancellable_cancel (entry->priv->cancellable);
         g_object_unref (entry->priv->cancellable);
         entry->priv->cancellable = NULL;
-        gtk_entry_completion_delete_action (completion, 0);
     }
 
-    gtk_entry_completion_set_match_func (gtk_entry_get_completion (GTK_ENTRY (entry)), matcher, NULL, NULL);
-    gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (entry)), entry->priv->model);
+    gtk_entry_completion_set_match_func (completion, matcher, NULL, NULL);
+    gtk_entry_completion_set_model (completion, entry->priv->model);
 
-    text = gtk_entry_get_text (GTK_ENTRY (entry));
+    text = gtk_editable_get_text (GTK_EDITABLE (entry));
 
-    if (text && *text)
-        entry->priv->custom_text = TRUE;
-    else
+    if (!text || *text == '\0')
         set_location_internal (entry, NULL, NULL, NULL);
 }
 
+static void
+set_entry_text (GisLocationEntry *entry,
+                const char       *text)
+{
+    GisLocationEntryPrivate *priv = entry->priv;
+
+    if (g_strcmp0 (gtk_editable_get_text (GTK_EDITABLE (priv->entry)), text) != 0)
+        gtk_editable_set_text (GTK_EDITABLE (priv->entry), text);
+}
+
 static void
 set_location_internal (GisLocationEntry *entry,
                        GtkTreeModel     *model,
@@ -311,24 +354,25 @@ set_location_internal (GisLocationEntry *entry,
 
     g_assert (iter == NULL || loc == NULL);
 
+    g_signal_handlers_block_by_func (entry, entry_changed, NULL);
+
     if (iter) {
         gtk_tree_model_get (model, iter,
                             LOC_GIS_LOCATION_ENTRY_COL_DISPLAY_NAME, &name,
                             LOC_GIS_LOCATION_ENTRY_COL_LOCATION, &priv->location,
                             -1);
-        gtk_entry_set_text (GTK_ENTRY (entry), name);
-        priv->custom_text = FALSE;
+        set_entry_text (entry, name);
         g_free (name);
     } else if (loc) {
         priv->location = g_object_ref (loc);
-        gtk_entry_set_text (GTK_ENTRY (entry), gweather_location_get_name (loc));
-        priv->custom_text = FALSE;
+        set_entry_text (entry, gweather_location_get_name (loc));
     } else {
         priv->location = NULL;
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
-        priv->custom_text = TRUE;
+        set_entry_text (entry, "");
     }
 
+    g_signal_handlers_unblock_by_func (entry, entry_changed, NULL);
+
     gtk_editable_set_position (GTK_EDITABLE (entry), -1);
     g_object_notify (G_OBJECT (entry), "location");
 }
@@ -355,7 +399,7 @@ gis_location_entry_set_location (GisLocationEntry *entry,
 
     g_return_if_fail (GIS_IS_LOCATION_ENTRY (entry));
 
-    completion = gtk_entry_get_completion (GTK_ENTRY (entry));
+    completion = gtk_entry_get_completion (GTK_ENTRY (entry->priv->entry));
     model = gtk_entry_completion_get_model (completion);
 
     if (loc == NULL) {
@@ -401,25 +445,6 @@ gis_location_entry_get_location (GisLocationEntry *entry)
         return NULL;
 }
 
-/**
- * gis_location_entry_has_custom_text:
- * @entry: a #GisLocationEntry
- *
- * Checks whether or not @entry's text has been modified by the user.
- * Note that this does not mean that no location is associated with @entry.
- * gis_location_entry_get_location() should be used for this.
- *
- * Return value: %TRUE if @entry's text was modified by the user, or %FALSE if
- * it's set to the default text of a location.
- **/
-gboolean
-gis_location_entry_has_custom_text (GisLocationEntry *entry)
-{
-    g_return_val_if_fail (GIS_IS_LOCATION_ENTRY (entry), FALSE);
-
-    return entry->priv->custom_text;
-}
-
 /**
  * gis_location_entry_set_city:
  * @entry: a #GisLocationEntry
@@ -446,7 +471,7 @@ gis_location_entry_set_city (GisLocationEntry *entry,
     g_return_val_if_fail (GIS_IS_LOCATION_ENTRY (entry), FALSE);
     g_return_val_if_fail (code != NULL, FALSE);
 
-    completion = gtk_entry_get_completion (GTK_ENTRY (entry));
+    completion = gtk_entry_get_completion (GTK_ENTRY (entry->priv->entry));
     model = gtk_entry_completion_get_model (completion);
 
     gtk_tree_model_get_iter_first (model, &iter);
@@ -757,6 +782,8 @@ fill_store (gpointer data, gpointer user_data)
     normalized = g_utf8_normalize (display_name, -1, G_NORMALIZE_ALL);
     compare_name = g_utf8_casefold (normalized, -1);
 
+    g_debug ("Adding geocode match %s", display_name);
+
     gtk_list_store_insert_with_values (user_data, NULL, -1,
                                        PLACE_GIS_LOCATION_ENTRY_COL_PLACE, place,
                                        PLACE_GIS_LOCATION_ENTRY_COL_DISPLAY_NAME, display_name,
@@ -773,57 +800,52 @@ _got_places (GObject      *source_object,
              GAsyncResult *result,
              gpointer      user_data)
 {
-    GList *places;
-    GisLocationEntry *self = user_data;
-    GError *error = NULL;
-    GtkListStore *store = NULL;
+    g_autolist(GeocodePlace) places = NULL;
+    GisLocationEntry *self = NULL;
+    g_autoptr(GError) error = NULL;
+    g_autoptr(GtkListStore) store = NULL;
     GtkEntryCompletion *completion;
 
     places = geocode_forward_search_finish (GEOCODE_FORWARD (source_object), result, &error);
-    if (places == NULL) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
         /* return without touching anything if cancelled (the entry might have been disposed) */
-        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-            g_clear_error (&error);
-            return;
-        }
-
-        g_clear_error (&error);
-        completion = gtk_entry_get_completion (user_data);
-        gtk_entry_completion_set_match_func (completion, matcher, NULL, NULL);
-        gtk_entry_completion_set_model (completion, self->priv->model);
-        goto out;
+        g_debug ("Geocode query cancelled");
+        return;
     }
 
-    completion = gtk_entry_get_completion (user_data);
-    store = gtk_list_store_new (5, G_TYPE_STRING, GEOCODE_TYPE_PLACE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store),
-                                             tree_compare_local_name, NULL, NULL);
-    g_list_foreach (places, fill_store, store);
-    g_list_free (places);
-    gtk_entry_completion_set_match_func (completion, new_matcher, NULL, NULL);
-    gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
-    g_object_unref (store);
+    self = GIS_LOCATION_ENTRY (user_data);
+    completion = gtk_entry_get_completion (GTK_ENTRY (self->priv->entry));
+
+    if (places == NULL) {
+        g_debug ("No geocode results, restoring default model");
+        gtk_entry_completion_set_match_func (completion, matcher, NULL, NULL);
+        gtk_entry_completion_set_model (completion, self->priv->model);
+    } else {
+        store = gtk_list_store_new (5, G_TYPE_STRING, GEOCODE_TYPE_PLACE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+        gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store),
+                                                 tree_compare_local_name, NULL, NULL);
+        g_list_foreach (places, fill_store, store);
+        gtk_entry_completion_set_match_func (completion, new_matcher, NULL, NULL);
+        gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
+    }
 
- out:
-    gtk_entry_completion_delete_action (completion, 0);
     g_clear_object (&self->priv->cancellable);
 }
 
 static void
 _no_matches (GtkEntryCompletion *completion, GisLocationEntry *entry) {
-    const gchar *key = gtk_entry_get_text(GTK_ENTRY (entry));
+    const gchar *key = gtk_editable_get_text (GTK_EDITABLE (entry->priv->entry));
     GeocodeForward *forward;
 
     if (entry->priv->cancellable) {
         g_cancellable_cancel (entry->priv->cancellable);
         g_object_unref (entry->priv->cancellable);
         entry->priv->cancellable = NULL;
-    } else {
-        gtk_entry_completion_insert_action_text (completion, 0, _("Loading…"));
     }
 
     entry->priv->cancellable = g_cancellable_new ();
 
+    g_debug ("Starting geocode query for %s", key);
     forward = geocode_forward_new_for_string(key);
     geocode_forward_search_async (forward, entry->priv->cancellable, _got_places, entry);
 }
